(**********************************************************************)
(** - Copyright (C) 1991 Lehrstuhl III fuer Informatik, RWTH Aachen - *)
(** This library is free software; you can redistribute it and/or     *)
(** modify it under the terms of the GNU Library General Public       *)
(** License as published by the Free Software Foundation; either      *)
(** version 2 of the License, or (at your option) any later version.  *)
(** See the file COPYING.LIB for details.                             *)
(**                                                                   *)
(** Author: Peter Klein                                               *)
(**********************************************************************)
INTERFACE InOut;

CONST
  EOL = '\012';
  (* Unix line separator *)


VAR Done: BOOLEAN;               (* Indicates prev operation's Success *)


PROCEDURE Read (VAR ch: CHAR);
  (** Done == NOT end-of-file on reading.  If true, ch is a  *
    * valid character, or is EOL. If false, ch is 0C and end *
    * of file has been reached.                              *)


PROCEDURE ReadString (VAR s: TEXT);
  (** Ignoring leading white space, read a sequence of chars *
    * not including blanks or control chars.  The char which *
    * follows the string is assigned to termCH.  It will not *
    * be seen in subsequent call to any Read routine.        *)


PROCEDURE ReadInt (VAR x: INTEGER);
  (** Combination of ReadString and conversion to INTEGER.     *
    * Syntax : ["+"|"-"] digit { digit } .  Leading blanks are *
    * ignored.  Variable termCH is set as in ReadString.       *
    * Done == "an integer was successfully read"               *)


PROCEDURE ReadCard (VAR x: CARDINAL);
  (** Combination of ReadString and conversion to CARDINAL.    *
    * Syntax : digit { digit } .  Leading blanks are ignored.  *
    * Variable termCH is set as in ReadString.                 *
    * Done == "a cardinal was successfully read"               *)


(* Output Procedures *)


PROCEDURE Write (ch: CHAR);
  (** Put ch to the output stream.  Write (EOL) is equivalent *
    * to WriteLn.  Other control characters have terminal-    *
    * dependent effects.                                      *)


PROCEDURE WriteLn ();
  (** Write a line separator character (EOL).  Under normal    *
    * conditions, this will send both carriage return and      *
    * line feed to tty device, or the EOL character to a file. *)


PROCEDURE WriteString (s: TEXT);
  (** Write the string s to the output stream.  Embedded EOL *
    * characters have the same effect as in Write.  Other    *
    * control characters have terminal-dependent effects.    *)


PROCEDURE WriteAOC (READONLY s: ARRAY OF CHAR);
  (** Write an ARRAY OF CHAR.                                *)


PROCEDURE WriteInt (x: INTEGER; n: CARDINAL);
  (** Write the integer x with at least n characters of width.  *
    * Leading zeroes are suppressed.  If n is greater than the  *
    * needed number of columns, the number is right-justified.  *)


PROCEDURE WriteCard (x: CARDINAL; n: CARDINAL);
  (** Write the cardinal x with at least n characters of width.*
    * Leading zeroes are suppressed.  If n is greater than the *
    * needed number of columns, the number is right-justified. *)


PROCEDURE ReadReal (VAR real: REAL; VAR success: BOOLEAN);

PROCEDURE WriteReal (real : REAL;
                     width: CARDINAL;  (* Width of field in chars *)
                     decPlaces: INTEGER (* neg->sci; 0->no point *));

END InOut.
