(**********************************************************************)
(** - Copyright (C) 1991 Lehrstuhl III fuer Informatik, RWTH Aachen - *)
(** This library is free software; you can redistribute it and/or     *)
(** modify it under the terms of the GNU Library General Public       *)
(** License as published by the Free Software Foundation; either      *)
(** version 2 of the License, or (at your option) any later version.  *)
(** See the file COPYING.LIB for details.                             *)
(**                                                                   *)
(** Author: Peter Klein                                               *)
(**********************************************************************)
MODULE InOut;

IMPORT Rd, Wr, Stdio, Fmt, Text, Scan;


PROCEDURE Read (VAR ch: CHAR) =
  BEGIN
    TRY ch := Rd.GetChar(Stdio.stdin); EXCEPT ELSE Done := FALSE; END;
  END Read;


PROCEDURE ReadString (VAR s: TEXT) =
  BEGIN
    TRY s := Rd.GetLine(Stdio.stdin); EXCEPT ELSE Done := FALSE; END;
  END ReadString;


PROCEDURE ReadInt (VAR x: INTEGER) =
  BEGIN
    TRY
      x := Scan.Int(Rd.GetLine(Stdio.stdin));
    EXCEPT
    ELSE
      Done := FALSE;
    END;
  END ReadInt;


PROCEDURE ReadCard (VAR x: CARDINAL) =
  VAR i: INTEGER;

  BEGIN
    ReadInt(i);
    x := i;
  END ReadCard;


PROCEDURE Write (ch: CHAR) =
  BEGIN
    TRY
      Wr.PutChar(Stdio.stdout, ch);
      Wr.Flush(Stdio.stdout);
    EXCEPT
    ELSE
      Done := FALSE;
    END;
  END Write;


PROCEDURE WriteLn () =
  BEGIN
    TRY
      Wr.PutChar(Stdio.stdout, '\n');
      Wr.Flush(Stdio.stdout);
    EXCEPT
    ELSE
      Done := FALSE;
    END;
  END WriteLn;


PROCEDURE WriteString (s: TEXT) =
  BEGIN
    TRY
      Wr.PutText(Stdio.stdout, s);
      Wr.Flush(Stdio.stdout);
    EXCEPT
    ELSE
      Done := FALSE;
    END;
  END WriteString;


PROCEDURE WriteAOC (READONLY s: ARRAY OF CHAR) =
  VAR
    t: TEXT;
    l: INTEGER;

  BEGIN
    t := Text.FromChars(s);
    l := Text.FindChar(t, '\000');
    IF (l >= 0) THEN t := Text.Sub(t, 0, l); END;
    WriteString(t);
  END WriteAOC;


PROCEDURE WriteInt (x: INTEGER; n: CARDINAL) =
  BEGIN
    TRY
      Wr.PutText(Stdio.stdout, Fmt.Pad(Fmt.Int(x), n));
      Wr.Flush(Stdio.stdout);
    EXCEPT
    ELSE
      Done := FALSE;
    END;
  END WriteInt;


PROCEDURE WriteCard (x: CARDINAL; n: CARDINAL) =
  BEGIN
    WriteInt(x, n);
  END WriteCard;


PROCEDURE ReadReal (VAR real: REAL; VAR success: BOOLEAN) =
  BEGIN
    success := TRUE;
    TRY
      real := Scan.Real(Rd.GetLine(Stdio.stdin));
    EXCEPT
    ELSE
      success := FALSE;
    END;
  END ReadReal;


PROCEDURE WriteReal (real: REAL; width: CARDINAL; decPlaces: INTEGER) =
  VAR style := Fmt.Style.AltFlo;

  BEGIN
    IF (decPlaces < 0) THEN
      style := Fmt.Style.AltSci;
      decPlaces := -decPlaces;
    ELSIF (decPlaces = 0) THEN
      style := Fmt.Style.Mix;
    END;

    TRY
      Wr.PutText(
        Stdio.stdout, Fmt.Pad(Fmt.Real(real, decPlaces, style), width));
    EXCEPT
    ELSE
      Done := FALSE;
    END;
  END WriteReal;


BEGIN
  Done := TRUE;
END InOut.
