(**********************************************************************)
(** - Copyright (C) 1991 Lehrstuhl III fuer Informatik, RWTH Aachen - *)
(** This library is free software; you can redistribute it and/or     *)
(** modify it under the terms of the GNU Library General Public       *)
(** License as published by the Free Software Foundation; either      *)
(** version 2 of the License, or (at your option) any later version.  *)
(** See the file COPYING.LIB for details.                             *)
(**                                                                   *)
(** Author: Peter Klein                                               *)
(**********************************************************************)
INTERFACE M2Dirs;

IMPORT Ctypes, Utypes;


TYPE
  Accessmode = {accessX,         (* executable by caller *)
                accessW,         (* writable by caller *)
                accessR          (* readable by caller *)
               };

  AccessSet = SET OF Accessmode;

  Permission = {permOX,          (* other execute *)
                permOW,          (* other write *)
                permOR,          (* other read *)
                permGX,          (* group execute *)
                permGW,          (* group write *)
                permGR,          (* group read *)
                permUX,          (* user execute *)
                permUW,          (* user write *)
                permUR,          (* user read *)
                permSGID,        (* set group id on execution *)
                permSUID         (* set user id on execution *)
               };

  PermissionSet = SET OF Permission;

  StatFileType = {statRegular,   (* regular file *)
                  statDirectory, (* directory *)
                  statCharacter, (* character special *)
                  statBlock,     (* block special *)
                  statSymLink,   (* symbolic link *)
                  statSocket     (* socket *)
                 };

  FStatType = RECORD             (* what FStat returns stuff into *)
                dev    : Utypes.dev_t;
                ino    : Utypes.ino_t;
                mode   : PermissionSet;
                nlink  : Ctypes.short;
                uid    : Ctypes.short;
                gid    : Ctypes.short;
                rdev   : Utypes.dev_t;
                size   : Utypes.off_t;
                atime  : Utypes.time_t;
                spare1 : Ctypes.int;
                mtime  : Utypes.time_t;
                spare2 : Ctypes.int;
                ctime  : Utypes.time_t;
                spare3 : Ctypes.int;
                blksize: Ctypes.long;
                blocks : Ctypes.long;
                spare4 : ARRAY [0 .. 1] OF Ctypes.long;
                type   : StatFileType;
              END;


CONST
  (* PermissionSet constants *)
  permAll = PermissionSet{Permission.permOX.. Permission.permUR};
  (* rwxrwxrwx or 0777 *)
  permNone = PermissionSet{};    (* no permissions *)

  (* AccessSet constant *)
  accessF = AccessSet{};         (* does file exist *)


PROCEDURE Access (name: TEXT; mode: AccessSet): INTEGER;
  (* determine existence and accessability of file *)


PROCEDURE Chdir (name: TEXT): INTEGER;
  (* change current working directory *)


PROCEDURE Chmod (name: TEXT; mode: PermissionSet): INTEGER;
  (* change access permissions of file *)


PROCEDURE Link (name1, name2: TEXT): INTEGER;
  (* make a (hard) file system link from existing file (name1) to new name
     (name2) *)


PROCEDURE Mkdir (name: TEXT; mode: PermissionSet): INTEGER;
  (* make a new directory *)


PROCEDURE Rename (old, new: TEXT): INTEGER;
  (* rename a file *)


PROCEDURE Rmdir (name: TEXT): INTEGER;
  (* delete a directory *)


PROCEDURE Stat (name: TEXT; VAR buf: FStatType): INTEGER;
  (* determine characteristics of named file *)


PROCEDURE Unlink (name: TEXT): INTEGER;
  (* remove link/file from file system *)

END M2Dirs.
